@@ -7,6 +7,8 @@ import logging
 
 LOCK_FILENAME = '/tmp/gpu-lock-magic-RaNdOM-This_Name-NEED_BE-THE_SAME-Across_Users'
 
+gpu_owner = None
+
 
 class NvidiasmiError(Exception):
     pass
@@ -17,9 +19,25 @@ def get_free_gpus():
 
         Note that this information is volatile.
     '''
-    raw_output = subprocess.check_output("nvidia-smi -q | grep 'Minor\|Processes' | grep -B1 'None' | tr -d ' ' | grep 'MinorNumber' | cut -d':' -f 2", shell=True)
+    gpus_smi = subprocess.check_output(["nvidia-smi", "--format=csv,noheader", "--query-gpu=index,gpu_bus_id"]).decode().strip()
+    processes_smi = subprocess.check_output(["nvidia-smi", "--format=csv,noheader", "--query-compute-apps=pid,gpu_bus_id"]).decode().strip()
 
-    return raw_output.decode().split()
+    gpus = {}
+    for line in gpus_smi.split('\n'):
+        idx, uuid = line.split(',')
+        gpus[uuid.strip()] = idx.strip()
+
+    # in case of no running processes
+    if processes_smi == '':
+        return list(gpus.values())
+
+    used_gpus = set()
+    for line in processes_smi.split('\n'):
+        _, uuid = line.split(',')
+        used_gpus.add(uuid.strip())
+
+    free_gpus = list(set(gpus.keys()) - used_gpus)
+    return sorted([gpus[free] for free in free_gpus])
 
 
 def pytorch_placeholder(device_no):
@@ -114,6 +132,8 @@ class GPUOwner:
                         gpus_to_allocate = free_gpus[:nb_gpus]
                         self.allocate_gpus(gpus_to_allocate)
 
+        self.devices_taken = [int(gpu) for gpu in gpus_to_allocate]
+
     def allocate_gpus(self, gpu_device_numbers):
         os.environ['CUDA_VISIBLE_DEVICES'] = ','.join(gpu_device_numbers)
         self.logger.info(f"Set CUDA_VISIBLE_DEVICES={os.environ['CUDA_VISIBLE_DEVICES']}")
@@ -125,3 +145,8 @@ class GPUOwner:
         except RuntimeError:
             self.logger.error('Failed to acquire placeholder, truly marvellous. Race condition with someone not using `GPUOwner?`')
             raise
+
+
+def claim_gpus(nb_gpus=1, placeholder_fn=pytorch_placeholder, logger=None, debug_sleep=0.0):
+    global gpu_owner
+    gpu_owner = GPUOwner(nb_gpus, placeholder_fn, logger, debug_sleep)
